# -*- coding: utf-8 -*-
# @author: Yong Yang
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
from scipy.ndimage import gaussian_filter
from astropy.io import fits
from multiprocessing import Pool
from itertools import product
def gauss(x,mu,sigma):    # define a Gaussian function
    return 1/(np.sqrt(2*np.pi)*sigma) * np.exp( -(x-mu)**2/(2*sigma**2) )
with fits.open('revised_gaia_halo_corr.fits') as file:    # read Gaia data file
    gaia = file[1].data
with fits.open('13Gyr-2.2dex.fits') as file:    # read isochrone file
    cmd = file[1].data
cmd.sort(order='Gmag')
ra_grid = np.arange(0,360+0.2,0.2)    # prepare for plotting all-sky map
dec_grid = np.arange(-90,90+0.2,0.2)
pmra_grid = np.arange(-15,15+1, 1)    # define StreamScanner variables grid
pmdec_grid = np.arange(-20,10+1, 1)
dm_grid = np.arange(11.5,17.5+0.2, 0.2)    # distance modulus
grids = list(product(pmra_grid, pmdec_grid, dm_grid))
def mf(paras):
    pmra,pmdec,dm = paras
    w_pmra = gauss(gaia['pmra'], mu= np.full(len(gaia), pmra), sigma= gaia['pmra_error'])
    w_pmdec = gauss(gaia['pmdec'], mu= np.full(len(gaia), pmdec), sigma= gaia['pmdec_error'])
    G = cmd['Gmag']+ dm    # move isochrone at dm
    w_cmd = gauss(gaia['GBP0']-gaia['GRP0'], 
        mu= np.interp(gaia['G0'], G, cmd['G_BPmag']-cmd['G_RPmag'],), 
        sigma= np.sqrt(gaia['phot_bp_mean_mag_error']**2+gaia['phot_rp_mean_mag_error']**2))
        # assign weights based on stars' color differences from the isochrone
    w = w_pmra*w_pmdec*w_cmd
    F, _, _ = np.histogram2d(gaia['ra'],gaia['dec'], bins=[ra_grid,dec_grid], weights=w, )
    F = np.rot90(F)
    FF = gaussian_filter(F, sigma=2)    # smooth weighted map
    fig,ax2 = plt.subplots(1,1,figsize=[12,8], )
    ax2.imshow(FF, cmap='gray', norm=matplotlib.colors.LogNorm(vmin=0.01,vmax=4), 
               extent=[0,360,-90,90], interpolation='None')
    ax2.set_xlabel(r'$\alpha$ ($^\circ$)')
    ax2.set_ylabel(r'$\delta$ ($^\circ$)')
    plt.tight_layout()
    fig.savefig('dm_pmra_pmdec_{}_{}_{}.png'.format(dm, pmra, pmdec,))
        # save figure to see whether there are stream structures by eye
    plt.close(fig)
p = Pool(20)
p.map_async(mf, grids)    # run in parallel
p.close()
p.join()
print('All Done !!')